
(******************************************************************************
  Apache portable runtime API for FreePascal

  @defgroup apr_allocator Internal Memory Allocation
  @ingroup APR 
  
  Note: I'm expect what it is very buggy... be careful ;)

  Zolotov Alex, 2007-2008
  zolotov-alex@shamangrad.net
  http://shamangrad.net/

 *****************************************************************************)
  

type
  {* the allocator structure  }
  apr_allocator_p = ^apr_allocator_t;
  apr_allocator_t = record end;

  {* the structure which holds information about the allocation  }
  {* basic memory node structure  }
  apr_memnode_p = ^apr_memnode_t;
  apr_memnode_t = record
    {* next memnode  }
    next: apr_memnode_p;
    {* reference to self  }
    ref: ^apr_memnode_p;
    {* size  }
    index: apr_uint32_t;
    {* how much free  }
    free_index: apr_uint32_t;
    {* pointer to first free memory  }
    first_avail: pchar;
    {* pointer to end of free memory  }
    endp: pchar;
  end;

  {* Symbolic constants  }

  const
     APR_ALLOCATOR_MAX_FREE_UNLIMITED = 0;
  {*
   * Create a new allocator
   * @param allocator The allocator we have just created.
   *
    }
function apr_allocator_create(out allocator: apr_allocator_p): apr_status_t; cdecl; external libapr name 'apr_allocator_create';

  {*
   * Destroy an allocator
   * @param allocator The allocator to be destroyed
   * @remark Any memnodes not given back to the allocator prior to destroying
   *         will _not_ be free()d.
    }
procedure apr_allocator_destroy(allocator: apr_allocator_p); cdecl; external libapr name 'apr_allocator_destroy';

  {*
   * Allocate a block of mem from the allocator
   * @param allocator The allocator to allocate from
   * @param size The size of the mem to allocate (excluding the
   *        memnode structure)
    }
function apr_allocator_alloc(allocator: apr_allocator_p; size: apr_size_t): apr_memnode_p; cdecl; external libapr name 'apr_allocator_destroy';

  {*
   * Free a block of mem, giving it back to the allocator
   * @param allocator The allocator to give the mem back to
   * @param memnode The memory node to return
    }
procedure apr_allocator_free(allocator: apr_allocator_p; memnode: apr_memnode_p); cdecl; external libapr name 'apr_allocator_free';

  {*
   * Set the owner of the allocator
   * @param allocator The allocator to set the owner for
   * @param pool The pool that is to own the allocator
   * @remark Typically pool is the highest level pool using the allocator
    }
  {
   * XXX: see if we can come up with something a bit better.  Currently
   * you can make a pool an owner, but if the pool doesn't use the allocator
   * the allocator will never be destroyed.
    }
procedure apr_allocator_owner_set(allocator: apr_allocator_p; pool: apr_pool_p); cdecl; external libapr name 'apr_allocator_owner_set';
procedure apr_allocator_set_owner(allocator: apr_allocator_p; pool: apr_pool_p); cdecl; external libapr name 'apr_allocator_set_owner';

  {*
   * Get the current owner of the allocator
   * @param allocator The allocator to get the owner from
    }
function apr_allocator_owner_get(allocator: apr_allocator_p): apr_pool_p; cdecl; external libapr name 'apr_allocator_owner_get';
function apr_allocator_get_owner(allocator: apr_allocator_p): apr_pool_p; cdecl; external libapr name 'apr_allocator_owner_get';

  {*
   * Set the current threshold at which the allocator should start
   * giving blocks back to the system.
   * @param allocator The allocator the set the threshold on
   * @param size The threshold.  0 == unlimited.
    }
procedure apr_allocator_max_free_set(allocator: apr_allocator_p; size: apr_size_t); cdecl; external libapr name 'apr_allocator_max_free_set';
procedure apr_allocator_set_max_free(allocator: apr_allocator_p; size: apr_size_t); cdecl; external libapr name 'apr_allocator_set_max_free';

  {*
   * Set a mutex for the allocator to use
   * @param allocator The allocator to set the mutex for
   * @param mutex The mutex
    }
procedure apr_allocator_mutex_set(allocator: apr_allocator_p; mutex: apr_thread_mutex_p); cdecl; external libapr name 'apr_allocator_mutex_set';
procedure apr_allocator_set_mutex(allocator: apr_allocator_p; mutex: apr_thread_mutex_p); cdecl; external libapr name 'apr_allocator_set_mutex';

  {*
   * Get the mutex currently set for the allocator
   * @param allocator The allocator
    }
function apr_allocator_mutex_get(allocator: apr_allocator_p): apr_thread_mutex_p; cdecl; external libapr name 'apr_allocator_mutex_get';
function apr_allocator_get_mutex(allocator: apr_allocator_p): apr_thread_mutex_p; cdecl; external libapr name 'apr_allocator_get_mutex';
