
(******************************************************************************
  Apache portable runtime API for FreePascal

  @defgroup apr_pools Memory Pool Functions
  @ingroup APR 
  
  Note: I'm expect what it is very buggy... be careful ;)

  Zolotov Alex, 2007-2008
  zolotov-alex@shamangrad.net
  http://shamangrad.net/

 *****************************************************************************)

  {*
   * @file apr_pools.h
   * @brief APR memory allocation
   *
   * Resource allocation routines...
   *
   * designed so that we don't have to keep track of EVERYTHING so that
   * it can be explicitly freed later (a fundamentally unsound strategy ---
   * particularly in the presence of die()).
   *
   * Instead, we maintain pools, and allocate items (both memory and I/O
   * handlers) from the pools --- currently there are two, one for per
   * transaction info, and one for config info.  When a transaction is over,
   * we can delete everything in the per-transaction apr_pool_t without fear,
   * and without thinking too hard about it either.
    }

  
    {
     * APR memory structure manipulators (pools, tables, and arrays).
      }
    {
     * Initialization
      }
    {*
     * Setup all of the internal structures required to use pools
     * @remark Programs do NOT need to call this directly.  APR will call this
     *      automatically from apr_initialize.
     * @internal
      }
function apr_pool_initialize: apr_status_t; cdecl; external libapr name 'apr_pool_initialize';

    {*
     * Tear down all of the internal structures required to use pools
     * @remark Programs do NOT need to call this directly.  APR will call this
     *      automatically from apr_terminate.
     * @internal
      }
procedure apr_pool_terminate; cdecl; external libapr name 'apr_pool_terminate';

    {*
     * Create a new pool.
     * @param newpool The pool we have just created.
     * @param parent The parent pool.  If this is NULL, the new pool is a root
     *        pool.  If it is non-NULL, the new pool will inherit all
     *        of its parent pool's attributes, except the apr_pool_t will
     *        be a sub-pool.
     * @param abort_fn A function to use if the pool cannot allocate more memory.
     * @param allocator The allocator to use with the new pool.  If NULL the
     *        allocator of the parent pool will be used.
      }
function apr_pool_create_ex(
	out newpool: apr_pool_p;
	parent: apr_pool_p;
	abort_fn: apr_abortfunc_t;
	allocator: apr_allocator_p
	): apr_status_t; cdecl; external libapr name 'apr_pool_create_ex';

    {*
     * Debug version of apr_pool_create_ex.
     * @param newpool @see apr_pool_create.
     * @param parent @see apr_pool_create.
     * @param abort_fn @see apr_pool_create.
     * @param allocator @see apr_pool_create.
     * @param file_line Where the function is called from.
     *        This is usually APR_POOL__FILE_LINE__.
     * @remark Only available when APR_POOL_DEBUG is defined.
     *         Call this directly if you have you apr_pool_create_ex
     *         calls in a wrapper function and wish to override
     *         the file_line argument to reflect the caller of
     *         your wrapper function.  If you do not have
     *         apr_pool_create_ex in a wrapper, trust the macro
     *         and don't call apr_pool_create_ex_debug directly.
      }
function apr_pool_create_ex_debug(
	out newpool: apr_pool_p;
	parent: apr_pool_p;
	abort_fn: apr_abortfunc_t;
	allocator: apr_allocator_p
	): apr_status_t; cdecl; external libapr name 'apr_pool_create_ex_debug';

    {*
     * Find the pools allocator
     * @param pool The pool to get the allocator from.
      }
function apr_pool_allocator_get(pool: apr_pool_p): apr_allocator_p; cdecl; external libapr name 'apr_pool_allocator_get';

    {*
     * Clear all memory in the pool and run all the cleanups. This also destroys all
     * subpools.
     * @param p The pool to clear
     * @remark This does not actually free the memory, it just allows the pool
     *         to re-use this memory for the next allocation.
     * @see apr_pool_destroy()
      }
procedure apr_pool_clear(pool: apr_pool_p); cdecl; external libapr name 'apr_pool_clear';

    {*
     * Debug version of apr_pool_clear.
     * @param p See: apr_pool_clear.
     * @param file_line Where the function is called from.
     *        This is usually APR_POOL__FILE_LINE__.
     * @remark Only available when APR_POOL_DEBUG is defined.
     *         Call this directly if you have you apr_pool_clear
     *         calls in a wrapper function and wish to override
     *         the file_line argument to reflect the caller of
     *         your wrapper function.  If you do not have
     *         apr_pool_clear in a wrapper, trust the macro
     *         and don't call apr_pool_destroy_clear directly.
      }
procedure apr_pool_clear_debug(pool: apr_pool_p; file_line: pchar); cdecl; external libapr name 'apr_pool_clear_debug';

    {*
     * Destroy the pool. This takes similar action as apr_pool_clear() and then
     * frees all the memory.
     * @param p The pool to destroy
     * @remark This will actually free the memory
      }
procedure apr_pool_destroy(pool: apr_pool_p); cdecl; external libapr name 'apr_pool_destroy';

    {*
     * Debug version of apr_pool_destroy.
     * @param p See: apr_pool_destroy.
     * @param file_line Where the function is called from.
     *        This is usually APR_POOL__FILE_LINE__.
     * @remark Only available when APR_POOL_DEBUG is defined.
     *         Call this directly if you have you apr_pool_destroy
     *         calls in a wrapper function and wish to override
     *         the file_line argument to reflect the caller of
     *         your wrapper function.  If you do not have
     *         apr_pool_destroy in a wrapper, trust the macro
     *         and don't call apr_pool_destroy_debug directly.
      }
procedure apr_pool_destroy_debug(pool: apr_pool_p; file_line: pchar); cdecl; external libapr name 'apr_pool_destroy_debug';

    {
     * Memory allocation
      }
    {*
     * Allocate a block of memory from a pool
     * @param p The pool to allocate from
     * @param size The amount of memory to allocate
     * @return The allocated memory
      }
function apr_palloc(pool: apr_pool_p; size: apr_size_t): pointer; cdecl; external libapr name 'apr_palloc';

    {*
     * Debug version of apr_palloc
     * @param p See: apr_palloc
     * @param size See: apr_palloc
     * @param file_line Where the function is called from.
     *        This is usually APR_POOL__FILE_LINE__.
     * @return See: apr_palloc
      }
function apr_palloc_debug(pool: apr_pool_p; size: apr_size_t; file_line: pchar): pointer; cdecl; external libapr name 'apr_palloc_debug';

    {
     * Pool Properties
      }
    {*
     * Set the function to be called when an allocation failure occurs.
     * @remark If the program wants APR to exit on a memory allocation error,
     *      then this function can be called to set the callback to use (for
     *      performing cleanup and then exiting). If this function is not called,
     *      then APR will return an error and expect the calling program to
     *      deal with the error accordingly.
      }
procedure apr_pool_abort_set(abort_fn: apr_abortfunc_t; pool: apr_pool_p); cdecl; external libapr name 'apr_pool_abort_set';
procedure apr_pool_set_abort(abort_fn: apr_abortfunc_t; pool: apr_pool_p); cdecl; external libapr name 'apr_pool_set_abort';

    {*
     * Get the abort function associated with the specified pool.
     * @param pool The pool for retrieving the abort function.
     * @return The abort function for the given pool.
      }
function apr_pool_abort_get(pool: apr_pool_p): apr_abortfunc_t; cdecl; external libapr name 'apr_pool_abort_get';
function apr_pool_get_abort(pool: apr_pool_p): apr_abortfunc_t; cdecl; external libapr name 'apr_pool_get_abort';

    {*
     * Get the parent pool of the specified pool.
     * @param pool The pool for retrieving the parent pool.
     * @return The parent of the given pool.
      }
function apr_pool_parent_get(pool: apr_pool_p): apr_pool_p; cdecl; external libapr name 'apr_pool_parent_get';
function apr_pool_get_parent(pool: apr_pool_p): apr_pool_p; cdecl; external libapr name 'apr_pool_get_parent';

    {*
     * Determine if pool a is an ancestor of pool b
     * @param a The pool to search
     * @param b The pool to search for
     * @return True if a is an ancestor of b, NULL is considered an ancestor
     *         of all pools.
      }
function apr_pool_is_ancestor(a, b: apr_pool_p): longbool; cdecl; external libapr name 'apr_pool_is_ancestor';

    {*
     * Tag a pool (give it a name)
     * @param pool The pool to tag
     * @param tag  The tag
      }
procedure apr_pool_tag(pool: apr_pool_p; tag: pchar); cdecl; external libapr name 'apr_pool_tag';

    {
     * User data management
      }
    {*
     * Set the data associated with the current pool
     * @param data The user data associated with the pool.
     * @param key The key to use for association
     * @param cleanup The cleanup program to use to cleanup the data (NULL if none)
     * @param pool The current pool
     * @warning The data to be attached to the pool should have a life span
     *          at least as long as the pool it is being attached to.
     *
     *      Users of APR must take EXTREME care when choosing a key to
     *      use for their data.  It is possible to accidentally overwrite
     *      data by choosing a key that another part of the program is using.
     *      Therefore it is advised that steps are taken to ensure that unique
     *      keys are used for all of the userdata objects in a particular pool
     *      (the same key in two different pools or a pool and one of its
     *      subpools is okay) at all times.  Careful namespace prefixing of
     *      key names is a typical way to help ensure this uniqueness.
      }
type
  cleanup_func = function (p: pointer): apr_status_t; cdecl;
  
function apr_pool_userdata_set(
	data: pointer;
	key: pchar;
	cleanup: cleanup_func;
	pool: apr_pool_p
	): apr_status_t; cdecl; external libapr name 'apr_pool_userdata_set';

    {*
     * Set the data associated with the current pool
     * @param data The user data associated with the pool.
     * @param key The key to use for association
     * @param cleanup The cleanup program to use to cleanup the data (NULL if none)
     * @param pool The current pool
     * @note same as apr_pool_userdata_set(), except that this version doesn't
     *       make a copy of the key (this function is useful, for example, when
     *       the key is a string literal)
     * @warning This should NOT be used if the key could change addresses by
     *       any means between the apr_pool_userdata_setn() call and a
     *       subsequent apr_pool_userdata_get() on that key, such as if a
     *       static string is used as a userdata key in a DSO and the DSO could
     *       be unloaded and reloaded between the _setn() and the _get().  You
     *       MUST use apr_pool_userdata_set() in such cases.
     * @warning More generally, the key and the data to be attached to the
     *       pool should have a life span at least as long as the pool itself.
     *
      }
function apr_pool_userdata_setn(
	data: pointer;
	key: pchar;
	cleanup: cleanup_func;
	pool: apr_pool_p
	): apr_status_t; cdecl; external libapr name 'apr_pool_userdata_setn';

    {*
     * Return the data associated with the current pool.
     * @param data The user data associated with the pool.
     * @param key The key for the data to retrieve
     * @param pool The current pool.
      }
function apr_pool_userdata_get(out data: pointer; out key: pchar; pool: apr_pool_p): apr_status_t; cdecl; external libapr name 'apr_pool_userdata_get';

    {
     * Cleanup
     *
     * Cleanups are performed in the reverse order they were registered.  That is:
     * Last In, First Out.
      }
    {*
     * Register a function to be called when a pool is cleared or destroyed
     * @param p The pool register the cleanup with
     * @param data The data to pass to the cleanup function.
     * @param plain_cleanup The function to call when the pool is cleared
     *                      or destroyed
     * @param child_cleanup The function to call when a child process is being
     *                      shutdown - this function is called in the child, obviously!
      }
procedure apr_pool_cleanup_register(
	pool: apr_pool_p;
	data: pointer;
	plain_cleanup: cleanup_func;
	child_cleanup: cleanup_func
	); cdecl; external libapr name 'apr_pool_cleanup_register';

    {*
     * Remove a previously registered cleanup function
     * @param p The pool remove the cleanup from
     * @param data The data to remove from cleanup
     * @param cleanup The function to remove from cleanup
     * @remarks For some strange reason only the plain_cleanup is handled by this
     *          function
      }
procedure apr_pool_cleanup_kill(pool: apr_pool_p; data: pointer; cleanup: cleanup_func); cdecl; external libapr name 'apr_pool_cleanup_kill';

    {*
     * Replace the child cleanup of a previously registered cleanup
     * @param p The pool of the registered cleanup
     * @param data The data of the registered cleanup
     * @param plain_cleanup The plain cleanup function of the registered cleanup
     * @param child_cleanup The function to register as the child cleanup
      }
procedure apr_pool_child_cleanup_set(
	pool: apr_pool_p;
	data: pointer;
	plain_cleanup: cleanup_func;
	child_cleanup: cleanup_func
	); cdecl; external libapr name 'apr_pool_child_cleanup_set';

    {*
     * Run the specified cleanup function immediately and unregister it. Use
     * @a data instead of the data that was registered with the cleanup.
     * @param p The pool remove the cleanup from
     * @param data The data to remove from cleanup
     * @param cleanup The function to remove from cleanup
      }
function apr_pool_cleanup_run(
	pool: apr_pool_p;
	data: pointer;
	cleanup: cleanup_func
	): apr_status_t; cdecl; external libapr name 'apr_pool_cleanup_run';

    {*
     * An empty cleanup function
     * @param data The data to cleanup
      }
(* error 
APR_DECLARE_NONSTD(apr_status_t) apr_pool_cleanup_null(void *data);
 in declarator_list *)
    { Preparing for exec() --- close files, etc., but *don't* flush I/O
     * buffers, *don't* wait for subprocesses, and *don't* free any memory.
      }
    {*
     * Run all of the child_cleanups, so that any unnecessary files are
     * closed because we are about to exec a new program
      }
procedure apr_pool_cleanup_for_exec; cdecl; external libapr name 'apr_pool_cleanup_for_exec';

    {*
     * @defgroup PoolDebug Pool Debugging functions.
     *
     * pools have nested lifetimes -- sub_pools are destroyed when the
     * parent pool is cleared.  We allow certain liberties with operations
     * on things such as tables (and on other structures in a more general
     * sense) where we allow the caller to insert values into a table which
     * were not allocated from the table's pool.  The table's data will
     * remain valid as long as all the pools from which its values are
     * allocated remain valid.
     *
     * For example, if B is a sub pool of A, and you build a table T in
     * pool B, then it's safe to insert data allocated in A or B into T
     * (because B lives at most as long as A does, and T is destroyed when
     * B is cleared/destroyed).  On the other hand, if S is a table in
     * pool A, it is safe to insert data allocated in A into S, but it
     * is *not safe* to insert data allocated from B into S... because
     * B can be cleared/destroyed before A is (which would leave dangling
     * pointers in T's data structures).
     *
     * In general we say that it is safe to insert data into a table T
     * if the data is allocated in any ancestor of T's pool.  This is the
     * basis on which the APR_POOL_DEBUG code works -- it tests these ancestor
     * relationships for all data inserted into tables.  APR_POOL_DEBUG also
     * provides tools (apr_pool_find, and apr_pool_is_ancestor) for other
     * folks to implement similar restrictions for their own data
     * structures.
     *
     * However, sometimes this ancestor requirement is inconvenient --
     * sometimes we're forced to create a sub pool (such as through
     * apr_sub_req_lookup_uri), and the sub pool is guaranteed to have
     * the same lifetime as the parent pool.  This is a guarantee implemented
     * by the *caller*, not by the pool code.  That is, the caller guarantees
     * they won't destroy the sub pool individually prior to destroying the
     * parent pool.
     *
     * In this case the caller must call apr_pool_join() to indicate this
     * guarantee to the APR_POOL_DEBUG code.  There are a few examples spread
     * through the standard modules.
     *
     * These functions are only implemented when #APR_POOL_DEBUG is set.
     *
     * @
      }

    {*
     * Guarantee that a subpool has the same lifetime as the parent.
     * @param p The parent pool
     * @param sub The subpool
      }
procedure apr_pool_join(pool, sub: apr_pool_p); cdecl; external libapr name 'apr_pool_join';

    {*
     * Find a pool from something allocated in it.
     * @param mem The thing allocated in the pool
     * @return The pool it is allocated in
      }
function apr_pool_find(mem: pointer): apr_pool_p; cdecl; external libapr name 'apr_pool_find';

    {*
     * Report the number of bytes currently in the pool
     * @param p The pool to inspect
     * @param recurse Recurse/include the subpools' sizes
     * @return The number of bytes
      }
function apr_pool_num_bytes(pool: apr_pool_p; recurse: longbool): apr_size_t; cdecl; external libapr name 'apr_pool_num_bytes';

    {*
     * Lock a pool
     * @param pool The pool to lock
     * @param flag  The flag
      }
procedure apr_pool_lock(pool: apr_pool_p; flag: longint); cdecl; external libapr name 'apr_pool_lock';
