
(******************************************************************************
  Apache portable runtime API for FreePascal

  @defgroup apr_thread_mutex Thread Mutex Routines
  @ingroup APR 
  
  Note: I'm expect what it is very buggy... be careful ;)

  Zolotov Alex, 2007-2008
  zolotov-alex@shamangrad.net
  http://shamangrad.net/

 *****************************************************************************)

type
  {* Opaque thread-local mutex structure  }
  apr_thread_mutex_p = ^apr_thread_mutex_t;
  apr_thread_mutex_t = record end;

const
  {* platform-optimal lock behavior  }
  APR_THREAD_MUTEX_DEFAULT = $0;
  {* enable nested (recursive) locks  }
  APR_THREAD_MUTEX_NESTED = $1;
  {* disable nested locks  }
  APR_THREAD_MUTEX_UNNESTED = $2;
  
  {*
   * Create and initialize a mutex that can be used to synchronize threads.
   * @param mutex the memory address where the newly created mutex will be
   *        stored.
   * @param flags Or'ed value of:
   * <PRE>
   *           APR_THREAD_MUTEX_DEFAULT   platform-optimal lock behavior.
   *           APR_THREAD_MUTEX_NESTED    enable nested (recursive) locks.
   *           APR_THREAD_MUTEX_UNNESTED  disable nested locks (non-recursive).
   * </PRE>
   * @param pool the pool from which to allocate the mutex.
   * @warning Be cautious in using APR_THREAD_MUTEX_DEFAULT.  While this is the
   * most optimial mutex based on a given platform's performance charateristics,
   * it will behave as either a nested or an unnested lock.
    }
function apr_thread_mutex_create(out mutex: apr_thread_mutex_p; flags: longword; pool: apr_pool_p): apr_status_t; cdecl; external libapr name 'apr_thread_mutex_create';

  {*
   * Acquire the lock for the given mutex. If the mutex is already locked,
   * the current thread will be put to sleep until the lock becomes available.
   * @param mutex the mutex on which to acquire the lock.
    }
function apr_thread_mutex_lock(mutex: apr_thread_mutex_p): apr_status_t; cdecl; external libapr name 'apr_thread_mutex_lock';

  {*
   * Attempt to acquire the lock for the given mutex. If the mutex has already
   * been acquired, the call returns immediately with APR_EBUSY. Note: it
   * is important that the APR_STATUS_IS_EBUSY(s) macro be used to determine
   * if the return value was APR_EBUSY, for portability reasons.
   * @param mutex the mutex on which to attempt the lock acquiring.
    }
function apr_thread_mutex_trylock(mutex: apr_thread_mutex_p): apr_status_t; cdecl; external libapr name 'apr_thread_mutex_trylock';

  {*
   * Release the lock for the given mutex.
   * @param mutex the mutex from which to release the lock.
    }
function apr_thread_mutex_unlock(mutex: apr_thread_mutex_p): apr_status_t; cdecl; external libapr name 'apr_thread_mutex_unlock';

  {*
   * Destroy the mutex and free the memory associated with the lock.
   * @param mutex the mutex to destroy.
    }
function apr_thread_mutex_destroy(mutex: apr_thread_mutex_p): apr_status_t; cdecl; external libapr name 'apr_thread_mutex_destroy';
