unit mxisqlite3;

{$IFDEF FPC}
  {$MODE OBJFPC}
  {$H+}
  {$assertions on}
{$ENDIF}

interface

uses mxsqlite3, SysUtils, Classes;

type
  EMXSQLiteException = class (Exception)
  end;
  
  TMXSQLite = class;
  
  TMXSQLiteBLOBStream = class (TStream)
    protected
      FActive: Boolean;
      FSQLite: TMXSQLite;
      FBLOBIO: sqlite3_blob_p;
      FOffset: Longword;
      FSize: Longword;
      function GetSize: Int64; override;
    public
      constructor Create(ASQLite: TMXSQLite; ABLOBIO: sqlite3_blob_p);
      destructor Destroy; override;
      
      function Read(var Buffer; Count: Longint): Longint; override;
      function Write(const Buffer; Count: Longint): Longint; override;
      function Seek(Offset: Longint; Origin: Word): Longint; override;
      //function Seek(const Offset: Int64; Origin: TSeekOrigin): Int64; override;
  end;
  
  TMXSQLiteStatement = class
    protected
      FActive: Boolean;
      FSQLite: TMXSQLite;
      FStmt: sqlite3_statement_p;
      FColumns: TStringList;
      
      //   
      procedure DoFinalize;
    public
      constructor Create(ASQLite: TMXSQLite; stmt: sqlite3_statement_p);
      destructor Destroy; override;
      
      //      
      function GetColumnCount: Integer;
      
      //   
      function GetColumnIndex(const Name: UTF8String): Integer;
      
      //   
      function GetColumnName(iCol: Integer): UTF8String;
      
      //     
      function GetInteger(iCol: Integer): Integer; overload;
      function GetInt64(iCol: Integer): Int64; overload;
      function GetFloat(iCol: Integer): Double; overload;
      function GetAnsiString(iCol: Integer): AnsiString; overload;
      function GetUTF8String(iCol: Integer): UTF8String; overload;
      function GetWideString(iCol: Integer): WideString; overload;
      function GetBLOB(iCol: Integer): AnsiString; overload;
      
      function GetInteger(const Name: UTF8String): Integer; overload;
      function GetInt64(const Name: UTF8String): Int64; overload;
      function GetFloat(const Name: UTF8String): Double; overload;
      function GetAnsiString(const Name: UTF8String): AnsiString; overload;
      function GetWideString(const Name: UTF8String): WideString; overload;
      function GetUTF8String(const Name: UTF8String): UTF8String; overload;
      function GetBLOB(const Name: UTF8String): AnsiString; overload;
      
      //     
      function GetParamCount: Integer;
      
      //     
      function GetParamName(Idx: Integer): UTF8String;
      
      //     
      function GetParamIndex(const Name: UTF8String): Integer;
      
      //   
      procedure SetParamNull(Idx: Integer); overload;
      procedure SetParamInteger(Idx: Integer; Value: Integer); overload;
      procedure SetParamInt64(Idx: Integer; Value: Int64); overload;
      procedure SetParamFloat(Idx: Integer; Value: Double); overload;
      procedure SetParamAnsiString(Idx: Integer; const Value: AnsiString); overload;
      procedure SetParamWideString(Idx: Integer; const Value: WideString); overload;
      procedure SetParamUTF8String(Idx: Integer; const Value: UTF8String); overload;
      procedure SetParamBLOB(Idx: Integer; const Value: AnsiString); overload;
      procedure SetParamZeroBLOB(Idx: Integer; Size: Integer); overload;
      
      procedure SetParamNull(const Name: UTF8String); overload;
      procedure SetParamInteger(const Name: UTF8String; Value: Integer); overload;
      procedure SetParamInt64(const Name: UTF8String; Value: Int64); overload;
      procedure SetParamFloat(const Name: UTF8String; Value: Double); overload;
      procedure SetParamAnsiString(const Name: UTF8String; const Value: AnsiString); overload;
      procedure SetParamWideString(const Name: UTF8String; const Value: WideString); overload;
      procedure SetParamUTF8String(const Name: UTF8String; const Value: UTF8String); overload;
      procedure SetParamBLOB(const Name: UTF8String; const Value: AnsiString); overload;
      procedure SetParamZeroBlob(const Name: UTF8String; Size: Integer); overload;
      
      function Next: Boolean;
      
      //     
      procedure Reset;
      
      //   -   
      //  ,     
      procedure Finalize;
      
      property SQLite: TMXSQLite read FSQLite;
      property ColumnCount: Integer read GetColumnCount;
      property ParamCount: Integer read GetParamCount;
  end;
  
  TMXSQLite = class
    protected
      FActive: Boolean;
      FFileName: UTF8String;
      FSQLite: sqlite3_p;
      FStatements: TList;
      
      procedure SetActive(Value: Boolean);
      procedure SetFileName(const Value: UTF8String);
      procedure RaiseException(const Message: string);
      procedure check(RetVal: Integer);
      
      //     
      procedure AddStatement(Stmt: TMXSQLiteStatement);
      
      //     
      procedure RemoveStatement(Stmt: TMXSQLiteStatement);
      
      //     
      procedure FinalizeStatements;
    public
      constructor Create;
      destructor Destroy; override;
      
      //   
      procedure Open;
      
      //   
      procedure Close;
      
      //    
      function Prepare(const SQL: UTF8String): TMXSQLiteStatement;
      
      //      
      procedure Execute(const SQL: UTF8String);
      
      function InsertId: Int64;
      
      //  
      function Escape(const Text: UTF8String): UTF8String;
      
      //       
      function Quote(const Text: UTF8String): UTF8String;
      
      //  
      procedure BeginTransaction;
      
      //  
      procedure Rollback;
      
      //  
      procedure Commit;
      
      //   BLOB
      function OpenBlob(const DB, Table, Field: UTF8String; RowId: Int64; RW: Boolean): TStream;
      
      // true -   , false - 
      property Active: Boolean read FActive;
      
      //     
      property FileName: UTF8String read FFileName write SetFileName;
  end;

implementation

constructor TMXSQLiteBLOBStream.Create(ASQLite: TMXSQLite; ABLOBIO: sqlite3_blob_p);
begin
  inherited Create;
  System.Assert(ASQLite <> nil);
  FActive := true;
  FSQLite := ASQLite;
  FBLOBIO := ABLOBIO;
  FOffset := 0;
  FSize := sqlite3_blob_bytes(ABLOBIO);
end;

destructor TMXSQLiteBLOBStream.Destroy;
begin
  System.Assert(FActive);
  FSQLite.check( sqlite3_blob_close(FBLOBIO) );
  inherited Destroy;
end;

function TMXSQLiteBLOBStream.GetSize: Int64;
begin
  Result := FSize;
end;

function TMXSQLiteBLOBStream.Read(var Buffer; Count: Longint): Longint;
begin
  if Count <= 0 then
  begin
    Result := 0;
    Exit;
  end; // if
  if FOffset + Longword(Count) > FSize then
  begin
    Count := FSize - FOffset;
  end; // if
  FSQLite.check( sqlite3_blob_read(FBLOBIO, @Buffer, Count, FOffset) );
  FOffset := FOffset + Count;
  Result := Count;
end;

function TMXSQLiteBLOBStream.Write(const Buffer; Count: Longint): Longint;
begin
  if Count <= 0 then
  begin
    Result := 0;
    Exit;
  end; // if
  if FOffset + Longword(Count) > FSize then
  begin
    Count := FSize - FOffset;
  end; // if
  FSQLite.check( sqlite3_blob_write(FBLOBIO, @Buffer, Count, FOffset) );
  FOffset := FOffset + Count;
  Result := Count;
end;

function TMXSQLiteBLOBStream.Seek(Offset: Longint; Origin: Word): Longint;
var NewPos: Int64;
begin
  case Origin of
  soFromBeginning: NewPos := Offset;
  soFromEnd: NewPos := FSize + Offset;
  soFromCurrent: NewPos := FOffset + Offset;
  end; // case
  System.Assert((NewPos > 0) and (NewPos <= FSize), 'TMXSQLiteBLOBStream.Seek failed');
  FOffset := NewPos;
  Result := FOffset;
end;

constructor TMXSQLiteStatement.Create(ASQLite: TMXSQLite; stmt: sqlite3_statement_p);
var
  i, Count: Integer;
begin
  inherited Create;
  System.Assert(ASQLite <> nil);
  FActive := true;
  FSQLite := ASQLite;
  FStmt :=  stmt;
  FColumns := TStringList.Create;
  
  //   
  Count := sqlite3_column_count(FStmt);
  for i := 0 to Count-1 do
  begin
    FColumns.Add(sqlite3_column_name(FStmt, i));
  end; // for i 
  FColumns.Sort;
end;

destructor TMXSQLiteStatement.Destroy;
begin
  //   Free  Destroy,  Finalize
  System.Assert(not FActive);
  FColumns.Free;
  inherited Destroy;
end;

function TMXSQLiteStatement.GetColumnCount: Integer;
begin
  Result := sqlite3_column_count(FStmt);
end;

function TMXSQLiteStatement.GetColumnIndex(const Name: UTF8String): Integer;
begin
  Result := FColumns.IndexOf(Name);
end;

function TMXSQLiteStatement.GetColumnName(iCol: Integer): UTF8String;
begin
  Result := sqlite3_column_name(FStmt, iCol);
end;

function TMXSQLiteStatement.GetInteger(iCol: Integer): Integer;
begin
  Result := sqlite3_column_int(FStmt, iCol);
end;

function TMXSQLiteStatement.GetInt64(iCol: Integer): Int64;
begin
  Result := sqlite3_column_int64(FStmt, iCol);
end;

function TMXSQLiteStatement.GetFloat(iCol: Integer): Double;
begin
  Result := sqlite3_column_double(FStmt, iCol);
end;

function TMXSQLiteStatement.GetAnsiString(iCol: Integer): AnsiString;
begin
  Result := GetWideString(iCol);
end;

function TMXSQLiteStatement.GetUTF8String(iCol: Integer): UTF8String;
var Len: Integer;
begin
  Len := sqlite3_column_bytes(FStmt, iCol);
  SetString(Result, sqlite3_column_text(FStmt, iCol), Len);
end;

function TMXSQLiteStatement.GetWideString(iCol: Integer): WideString;
var Len: Integer;
begin
  Len := sqlite3_column_bytes16(FStmt, iCol) div SizeOf(WideChar);
  SetString(Result, sqlite3_column_text16(FStmt, iCol), Len);
end;

function TMXSQLiteStatement.GetBLOB(iCol: Integer): AnsiString;
var Len: Integer;
begin
  Len := sqlite3_column_bytes(FStmt, iCol);
  SetString(Result, PChar(sqlite3_column_blob(FStmt, iCol)), Len);
end;

function TMXSQLiteStatement.GetInteger(const Name: UTF8String): Integer;
begin
  Result := GetInteger(GetColumnIndex(Name));
end;

function TMXSQLiteStatement.GetInt64(const Name: UTF8String): Int64;
begin
  Result := GetInt64(GetColumnIndex(Name));
end;

function TMXSQLiteStatement.GetFloat(const Name: UTF8String): Double;
begin
  Result := GetFloat(GetColumnIndex(Name));
end;

function TMXSQLiteStatement.GetAnsiString(const Name: UTF8String): AnsiString;
begin
  Result := GetAnsiString(GetColumnIndex(Name));
end;

function TMXSQLiteStatement.GetWideString(const Name: UTF8String): WideString;
begin
  Result := GetWideString(GetColumnIndex(Name));
end;

function TMXSQLiteStatement.GetUTF8String(const Name: UTF8String): UTF8String;
begin
  Result := GetUTF8String(GetColumnIndex(Name));
end;

function TMXSQLiteStatement.GetBLOB(const Name: UTF8String): AnsiString;
begin
  Result := GetBLOB(GetColumnIndex(Name));
end;

function TMXSQLiteStatement.GetParamCount: Integer;
begin
  Result := sqlite3_bind_parameter_count(FStmt);
end;

function TMXSQLiteStatement.GetParamName(Idx: Integer): UTF8String;
begin
  Result := sqlite3_bind_parameter_name(FStmt, Idx + 1);
end;

function TMXSQLiteStatement.GetParamIndex(const Name: UTF8String): Integer;
begin
  Result := sqlite3_bind_parameter_index(FStmt, PChar(Name)) - 1;
end;

procedure TMXSQLiteStatement.SetParamNull(Idx: Integer);
begin
  FSQLite.check(sqlite3_bind_null(FStmt, Idx + 1));
end;

procedure TMXSQLiteStatement.SetParamInteger(Idx: Integer; Value: Integer);
begin
  FSQLite.check(sqlite3_bind_int(FStmt, Idx + 1, Value));
end;

procedure TMXSQLiteStatement.SetParamInt64(Idx: Integer; Value: Int64);
begin
  FSQLite.check(sqlite3_bind_int64(FStmt, Idx + 1, Value));
end;

procedure TMXSQLiteStatement.SetParamFloat(Idx: Integer; Value: Double);
begin
  FSQLite.check(sqlite3_bind_double(FStmt, Idx + 1, Value));
end;

procedure TMXSQLiteStatement.SetParamAnsiString(Idx: Integer; const Value: AnsiString);
begin
  SetParamWideString(Idx, Value);
end;

procedure TMXSQLiteStatement.SetParamWideString(Idx: Integer; const Value: WideString);
begin
  FSQLite.check(sqlite3_bind_text16(FStmt, Idx + 1, PWideChar(Value),
    Length(Value) * SizeOf(WideChar), TDestructor(SQLITE_TRANSIENT)));
end;

procedure TMXSQLiteStatement.SetParamUTF8String(Idx: Integer; const Value: UTF8String);
begin
  FSQLite.check(sqlite3_bind_text(FStmt, Idx + 1, PChar(Value),
    Length(Value), TDestructor(SQLITE_TRANSIENT)));
end;

procedure TMXSQLiteStatement.SetParamBLOB(Idx: Integer; const Value: AnsiString);
begin
  FSQLite.check(sqlite3_bind_blob(FStmt, Idx + 1, PChar(Value),
    Length(Value), TDestructor(SQLITE_TRANSIENT)));
end;

procedure TMXSQLiteStatement.SetParamZeroBLOB(Idx: Integer; Size: Integer);
begin
  FSQLite.check( sqlite3_bind_zeroblob(FStmt, Idx + 1, Size) );
end;

procedure TMXSQLiteStatement.SetParamNull(const Name: UTF8String);
begin
  SetParamNull(GetParamIndex(Name));
end;

procedure TMXSQLiteStatement.SetParamInteger(const Name: UTF8String; Value: Integer);
begin
  SetParamInteger(GetParamIndex(Name), Value);
end;

procedure TMXSQLiteStatement.SetParamInt64(const Name: UTF8String; Value: Int64);
begin
  SetParamInt64(GetParamIndex(Name), Value);
end;

procedure TMXSQLiteStatement.SetParamFloat(const Name: UTF8String; Value: Double);
begin
  SetParamFloat(GetParamIndex(Name), Value);
end;

procedure TMXSQLiteStatement.SetParamAnsiString(const Name: UTF8String; const Value: AnsiString);
begin
  SetParamAnsiString(GetParamIndex(Name), Value);
end;

procedure TMXSQLiteStatement.SetParamWideString(const Name: UTF8String; const Value: WideString);
begin
  SetParamWideString(GetParamIndex(Name), Value);
end;

procedure TMXSQLiteStatement.SetParamUTF8String(const Name: UTF8String; const Value: UTF8String);
begin
  SetParamUTF8String(GetParamIndex(Name), Value);
end;

procedure TMXSQLiteStatement.SetParamBLOB(const Name: UTF8String; const Value: AnsiString);
begin
  SetParamBLOB(GetParamIndex(Name), Value);
end;

procedure TMXSQLiteStatement.SetParamZeroBlob(const Name: UTF8String; Size: Integer);
begin
  SetParamZeroBlob(GetParamIndex(Name), Size);
end;

function TMXSQLiteStatement.Next: Boolean;
var r: Integer;
begin
  r := sqlite3_step(FStmt);
  case r of
  SQLITE_ROW: Result := true;
  SQLITE_DONE: Result := false;
  else
    FSQLite.check(r);
    Result := false;
  end; // case
end;

procedure TMXSQLiteStatement.Reset;
begin
  FSQLite.check(sqlite3_reset(FStmt));
  FSQLite.check(sqlite3_clear_bindings(FStmt));
end;

procedure TMXSQLiteStatement.DoFinalize;
begin
  FSQLite.check( sqlite3_finalize(FStmt) );
end;

procedure TMXSQLiteStatement.Finalize;
begin
  // do not call Finalize twice
  // or other TSQLStatement routine after call to Finalize,
  // because FSQLDriver.RemoveStatement destroys object
  System.Assert(FActive);
  FSQLite.check( sqlite3_finalize(FStmt) );
  FActive := false;
  FSQLite.RemoveStatement(Self);
end;

constructor TMXSQLite.Create;
begin
  inherited Create;
  FActive := false;
  FSQLite := nil;
  FStatements := TList.Create;
end;

destructor TMXSQLite.Destroy;
begin
  Close;
  FStatements.Free;
  inherited Destroy;
end;

procedure TMXSQLite.SetActive(Value: Boolean);
begin
  if Value then Open else Close;
end;

procedure TMXSQLite.SetFileName(const Value: UTF8String);
begin
  FFileName := Value;
end;

procedure TMXSQLite.RaiseException(const Message: string);
begin
  raise EMXSQLiteException.Create(Message);
end;

procedure TMXSQLite.check(RetVal: Integer);
begin
  if RetVal <> SQLITE_OK then
  begin
    RaiseException('SQLite error #' + IntToStr(RetVal) + ': ' + sqlite3_errmsg(FSQLite) );
  end; // if
end;

procedure TMXSQLite.AddStatement(Stmt: TMXSQLiteStatement);
begin
  FStatements.Add(Stmt);
end;

procedure TMXSQLite.RemoveStatement(Stmt: TMXSQLiteStatement);
var Idx: Integer;
begin
  Idx := FStatements.IndexOf(Stmt);
  //     
  System.Assert(Idx >= 0);
  FStatements.Delete(Idx);
  Stmt.Free;
end;

procedure TMXSQLite.FinalizeStatements;
var
  i, Count: Integer;
  Stmt: TMXSQLiteStatement;
begin
  Count := FStatements.Count;
  for i := 0 to Count-1 do
  begin
    Stmt := TMXSQLiteStatement(FStatements[i]);
    Stmt.DoFinalize;
    Stmt.FActive := false;
    Stmt.Free;
  end; // for i
end;

procedure TMXSQLite.Open;
begin
  if not FActive then
  begin
    FActive := true;
    check( sqlite3_open(PChar(FFileName), FSQLite) );
  end; // if
end;

procedure TMXSQLite.Close;
begin
  if FActive then
  begin
    FinalizeStatements;
    check( sqlite3_close(FSQLite) );
    FActive := false;
  end; // if
end;

function TMXSQLite.Prepare(const SQL: UTF8String): TMXSQLiteStatement;
var
  stmt: sqlite3_statement_p;
  tail: PChar;
begin
  check( sqlite3_prepare(FSQLite, PChar(SQL), Length(SQL), stmt, tail) );
  Result := TMXSQLiteStatement.Create(Self, stmt);
  AddStatement(Result);
end;

procedure TMXSQLite.Execute(const SQL: UTF8String);
begin
  check( sqlite3_exec(FSQLite, PChar(SQL), nil, nil, nil) );
end;

function TMXSQLite.InsertId: Int64;
begin
  Result := sqlite3_last_insert_rowid(FSQLite);
end;

function TMXSQLite.Escape(const Text: UTF8String): UTF8String;
var
  p, vaList: PChar;
begin
  vaList := PChar(Text);
  p := sqlite3_vmprintf('%q', @vaList);
  Result := p;
  sqlite3_free(p);
end;

function TMXSQLite.Quote(const Text: UTF8String): UTF8String;
var
  p, vaList: PChar;
begin
  vaList := PChar(Text);
  p := sqlite3_vmprintf('''%q''', @vaList);
  Result := p;
  sqlite3_free(p);
end;

procedure TMXSQLite.BeginTransaction;
begin
  Execute('BEGIN');
end;

procedure TMXSQLite.Rollback;
begin
  Execute('ROLLBACK');
end;

procedure TMXSQLite.Commit;
begin
  Execute('COMMIT');
end;

function TMXSQLite.OpenBlob(const DB, Table, Field: UTF8String; RowId: Int64; RW: Boolean): TStream;
var BLOBIO: sqlite3_blob_p;
begin
  check( sqlite3_blob_open(FSQLite, PChar(DB), PChar(Table), PChar(Field), RowId, Ord(RW), BLOBIO) );
  Result := TMXSQLiteBLOBStream.Create(Self, BLOBIO);
end;

end.
