program test;

{$IFDEF FPC}
  {$MODE OBJFPC}
  {$H+}
{$ENDIF}

uses mxisqlite3, SysUtils, Classes;

procedure ShowHelp;
// показать подсказку
begin
  writeln('test create <db file name> - Create database');
  writeln('test list <db> - list files in database');
  writeln('test put <db> <name> <file> - put file into database');
  writeln('test get <db> <name> <file> - get file from database');
end;

procedure CreateBase(const FileName: string);
// Создать базу
var sq: TMXSQLite;
begin
  if ParamCount < 2 then
  begin
    writeln('test create <db file name>');
    Exit;
  end; // if
  sq := TMXSQLite.Create;
  try
    sq.FileName := FileName;
    sq.Open;
    sq.Execute('DROP TABLE IF EXISTS test;');
    sq.Execute('CREATE TABLE test(id INTEGER PRIMARY KEY AUTOINCREMENT, name VARCHAR(255) UNIQUE, content  BLOB);');
  finally
    sq.Free;
  end;
end;

procedure ListFiles(const DB: string);
// показать список файлов в БД
var
  sq: TMXSQLite;
  stmt: TMXSQLiteStatement;
begin
  if ParamCount < 2 then
  begin
    writeln('test list <db>');
    Exit;
  end; // if
  sq := TMXSQLite.Create;
  try
    sq.FileName := DB;
    sq.Open;
    stmt := sq.Prepare('SELECT name, length(content) AS size FROM test');
    while stmt.Next do
    begin
      writeln(stmt.GetAnsiString('name'), ', ', stmt.GetInteger('size'), ' bytes');
    end; // while
  finally
    sq.Free;
  end;
end;

procedure PutFile(const DB, Name, FileName: string);
// записать файл в БД
var
  sq: TMXSQLite;
  stmt: TMXSQLiteStatement;
  f: file;
  i, count: Integer;
  Stream: TStream;
  buf: array [byte] of byte;
  rowid: Int64;
begin
  if ParamCount < 4 then
  begin
    writeln('test put <db> <name> <file name>');
    Exit;
  end; // if
  sq := TMXSQLite.Create;
  try
    sq.FileName := DB;
    sq.Open;
    
    // создадим BLOB нужного размера
    assign(f, FileName);
    reset(f,1);
    stmt := sq.Prepare('INSERT INTO test (name, content) VALUES ($name, $value);');
    stmt.SetParamAnsiString('$name', Name);
    stmt.SetParamZeroBLOB('$value', FileSize(f));
    stmt.Next;
    rowid := sq.InsertId;
    writeln(rowid);
    stmt.Finalize;
    
    // запишем данные
    stream := sq.OpenBlob('main', 'test', 'content', rowid, true);
    try
      for i := 0 to (FileSize(f) + SizeOf(buf) - 1) div SizeOf(buf) do
      begin
        BlockRead(f, buf, SizeOf(buf), count);
        Stream.WriteBuffer(buf, Count);
        write('.');
      end; // for i
    finally
      // обязательно нужно освободить поток перед выполнением других
      // операций с БД
      stream.Free;
      writeln;
    end; // try
    close(f);
    
  finally
    sq.Free;
  end;
end;

procedure GetFile(const DB, Name, FileName: string);
// извлеч файл из БД
var
  sq: TMXSQLite;
  stmt: TMXSQLiteStatement;
  f: file;
  i, count: Integer;
  Stream: TStream;
  buf: array [byte] of byte;
  rowid: Int64;
begin
  if ParamCount < 4 then
  begin
    writeln('test get <db> <name> <file name>');
    Exit;
  end; // if
  sq := TMXSQLite.Create;
  try
    sq.FileName := DB;
    sq.Open;
    
    stmt := sq.Prepare('SELECT id, name FROM test WHERE name = $name');
    stmt.SetParamAnsiString('$name', Name);
    if not stmt.Next then
    begin
      writeln('File "', Name, '" not found in database');
      Exit;
    end; // if
    rowid := stmt.GetInt64('id');
    stmt.Finalize;
    
    assign(f, FileName);
    rewrite(f, 1);
    stream := sq.OpenBlob('main', 'test', 'content', rowid, false);
    try
      for i := 0 to (Stream.Size + SizeOf(buf) - 1) div SizeOf(buf) do
      begin
        Count := Stream.Read(buf, SizeOf(buf));
        BlockWrite(f, buf, Count);
        write('.');
      end; // for i
    finally
      stream.free;
      writeln;
    end;
    close(f);
    
  finally
    sq.Free;
  end;
end;

begin
  try
    if (ParamCount < 1) or (ParamStr(1) = 'help') then ShowHelp
    else if ParamStr(1) = 'create' then CreateBase(ParamStr(2))
    else if ParamStr(1) = 'list' then ListFiles(ParamStr(2))
    else if ParamStr(1) = 'put' then PutFile(ParamStr(2), ParamStr(3), ParamStr(4))
    else if ParamStr(1) = 'get' then GetFile(ParamStr(2), ParamStr(3), ParamStr(4))
    else ShowHelp;
    writeln;
    writeln('ok');
  except
    on E: Exception do
    begin
      writeln;
      writeln(E.ClassName, ': ', E.Message);
    end;
  end; // try
end.
