
(******************************************************************************
    
  Subversion Client API for FreePascal

  @file svn_config.h
  @brief Accessing SVN configuration files.
     
  Note: I'm expect what it is very buggy... be careful ;)

  Zolotov Alex, 2007-2008
  zolotov-alex@shamangrad.net
  http://shamangrad.net/

 *****************************************************************************)

type
  psvn_config_t  = ^svn_config_t;
  svn_config_p = ^svn_config_t;
  svn_config_t = record end;

  {*************************************************************************
   ***                                                                    ***
   ***  For a description of the SVN configuration file syntax, see       ***
   ***  your ~/.subversion/README, which is written out automatically by  ***
   ***  svn_config_ensure().                                              ***
   ***                                                                    ***
   ************************************************************************* }
  {* Opaque structure describing a set of configuration options.  }

  {** Configuration Defines ** }
  {*
   * @name Client configuration files strings
   * Strings for the names of files, sections, and options in the
   * client configuration files.
   * @
    }

const
  SVN_CONFIG_CATEGORY_SERVERS = 'servers';
  SVN_CONFIG_SECTION_GROUPS = 'groups';
  SVN_CONFIG_SECTION_GLOBAL = 'global';
  SVN_CONFIG_OPTION_HTTP_PROXY_HOST = 'http-proxy-host';
  SVN_CONFIG_OPTION_HTTP_PROXY_PORT = 'http-proxy-port';
  SVN_CONFIG_OPTION_HTTP_PROXY_USERNAME = 'http-proxy-username';
  SVN_CONFIG_OPTION_HTTP_PROXY_PASSWORD = 'http-proxy-password';
  SVN_CONFIG_OPTION_HTTP_PROXY_EXCEPTIONS = 'http-proxy-exceptions';
  SVN_CONFIG_OPTION_HTTP_TIMEOUT = 'http-timeout';
  SVN_CONFIG_OPTION_HTTP_COMPRESSION = 'http-compression';
  SVN_CONFIG_OPTION_NEON_DEBUG_MASK = 'neon-debug-mask';
  SVN_CONFIG_OPTION_SSL_AUTHORITY_FILES = 'ssl-authority-files';
  SVN_CONFIG_OPTION_SSL_TRUST_DEFAULT_CA = 'ssl-trust-default-ca';
  SVN_CONFIG_OPTION_SSL_CLIENT_CERT_FILE = 'ssl-client-cert-file';
  SVN_CONFIG_OPTION_SSL_CLIENT_CERT_PASSWORD = 'ssl-client-cert-password';
  SVN_CONFIG_CATEGORY_CONFIG = 'config';
  SVN_CONFIG_SECTION_AUTH = 'auth';
  SVN_CONFIG_OPTION_STORE_PASSWORDS = 'store-passwords';
  SVN_CONFIG_OPTION_STORE_AUTH_CREDS = 'store-auth-creds';
  SVN_CONFIG_SECTION_HELPERS = 'helpers';
  SVN_CONFIG_OPTION_EDITOR_CMD = 'editor-cmd';
  SVN_CONFIG_OPTION_DIFF_CMD = 'diff-cmd';
  SVN_CONFIG_OPTION_DIFF3_CMD = 'diff3-cmd';
  SVN_CONFIG_OPTION_DIFF3_HAS_PROGRAM_ARG = 'diff3-has-program-arg';
  SVN_CONFIG_SECTION_MISCELLANY = 'miscellany';
  SVN_CONFIG_OPTION_GLOBAL_IGNORES = 'global-ignores';
  SVN_CONFIG_OPTION_LOG_ENCODING = 'log-encoding';
  SVN_CONFIG_OPTION_USE_COMMIT_TIMES = 'use-commit-times';
  SVN_CONFIG_OPTION_TEMPLATE_ROOT = 'template-root';
  SVN_CONFIG_OPTION_ENABLE_AUTO_PROPS = 'enable-auto-props';
  SVN_CONFIG_OPTION_NO_UNLOCK = 'no-unlock';
  SVN_CONFIG_SECTION_TUNNELS = 'tunnels';
  SVN_CONFIG_SECTION_AUTO_PROPS = 'auto-props';
  {* @  }
  {* @name Repository conf directory configuration files strings
   * Strings for the names of sections and options in the
   * repository conf directory configuration files.
   * @
    }
  { For repository svnserve.conf files  }
  SVN_CONFIG_SECTION_GENERAL = 'general';
  SVN_CONFIG_OPTION_ANON_ACCESS = 'anon-access';
  SVN_CONFIG_OPTION_AUTH_ACCESS = 'auth-access'; 
  SVN_CONFIG_OPTION_PASSWORD_DB = 'password-db';
  SVN_CONFIG_OPTION_REALM = 'realm';
  SVN_CONFIG_OPTION_AUTHZ_DB = 'authz-db';
  { For repository password database  }
  SVN_CONFIG_SECTION_USERS = 'users';
  {* @  }
  {** Configuration Default Values ** }
  SVN_CONFIG_DEFAULT_GLOBAL_IGNORES = '*.o *.lo *.la #*# .*.rej *.rej .*~ *~ .#* .DS_Store';
  SVN_CONFIG_TRUE = 'true'; 
  SVN_CONFIG_FALSE = 'false';
  {* Read configuration information from the standard sources and merge it
   * into the hash @a *cfg_hash.  If @a config_dir is not NULL it specifies a
   * directory from which to read the configuration files, overriding all
   * other sources.  Otherwise, first read any system-wide configurations
   * (from a file or from the registry), then merge in personal
   * configurations (again from file or registry).  The hash and all its data
   * are allocated in @a pool.
   *
   * @a *cfg_hash is a hash whose keys are @c const char * configuration
   * categories (@c SVN_CONFIG_CATEGORY_SERVERS,
   * @c SVN_CONFIG_CATEGORY_CONFIG, etc.) and whose values are the @c
   * svn_config_t * items representing the configuration values for that
   * category.  
    }

  function svn_config_get_config(var cfg_hash: papr_hash_t; config_dir:pchar; pool:apr_pool_p): psvn_error_t; cdecl; external libsvn_subr name 'svn_config_get_config';

  {* Read configuration data from @a file (a file or registry path) into
   * @a *cfgp, allocated in @a pool.
   *
   * If @a file does not exist, then if @a must_exist, return an error,
   * otherwise return an empty @c svn_config_t.
    }
  function svn_config_read(var cfgp: psvn_config_t; afile:pchar; must_exist:svn_boolean_t; pool:apr_pool_p): psvn_error_t; cdecl; external libsvn_subr name 'svn_config_read';

  {* Like svn_config_read(), but merges the configuration data from @a file
   * (a file or registry path) into @a *cfg, which was previously returned
   * from svn_config_read().  This function invalidates all value
   * expansions in @a cfg, so that the next svn_config_get() takes the
   * modifications into account.
    }
  function svn_config_merge(cfg:psvn_config_t; afile:pchar; must_exist:svn_boolean_t):psvn_error_t; cdecl; external libsvn_subr name 'svn_config_merge';

  {* Find the value of a (@a section, @a option) pair in @a cfg, set @a 
   * *valuep to the value.
   *
   * If @a cfg is @c NULL, just sets @a *valuep to @a default_value. If
   * the value does not exist, expand and return @a default_value. @a
   * default_value can be NULL.
   *
   * The returned value will be valid at least until the next call to
   * svn_config_get(), or for the lifetime of @a default_value. It is
   * safest to consume the returned value immediately.
   *
   * This function may change @a cfg by expanding option values.
    }
  procedure svn_config_get(cfg:psvn_config_t; valuep:Ppchar; section:pchar; option:pchar; default_value:pchar); cdecl; external libsvn_subr name 'svn_config_get';

  {* Add or replace the value of a (@a section, @a option) pair in @a cfg with 
   * @a value.
   *
   * This function invalidates all value expansions in @a cfg.
    }
  procedure svn_config_set(cfg:psvn_config_t; section:pchar; option:pchar; value:pchar); cdecl; external libsvn_subr name 'svn_config_set';

  {* Like svn_config_get(), but for boolean values.
   *
   * Parses the option as a boolean value. The recognized representations
   * are 'true'/'false', 'yes'/'no', 'on'/'off', '1'/'0'; case does not
   * matter. Returns an error if the option doesn't contain a known string.
    }
  function svn_config_get_bool(cfg:psvn_config_t; out valuep: svn_boolean_t; section: pchar; option: pchar; default_value: svn_boolean_t): svn_error_p; cdecl; external libsvn_subr name 'svn_config_get_bool';

  {* Like svn_config_set(), but for boolean values.
   *
   * Sets the option to 'true'/'false', depending on @a value.
    }
  procedure svn_config_set_bool(cfg:psvn_config_t; section:pchar; option:pchar; value:svn_boolean_t); cdecl; external libsvn_subr name 'svn_config_set_bool';

type
  {* Similar to @c svn_config_section_enumerator2_t, but is not
   * provided with a memory pool argument.
   *
   * See svn_config_enumerate_sections() for the details of this type.
   *
   * @deprecated Provided for backwards compatibility with the 1.2 API.
    }
  svn_config_section_enumerator_t = function (name: pchar; baton: pointer): svn_boolean_t; cdecl;
  {* Similar to svn_config_enumerate_sections2(), but uses a memory pool of 
   * @a cfg instead of one that is explicitely provided.
   *
   * @deprecated Provided for backwards compatibility with the 1.2 API. 
    }

  function svn_config_enumerate_sections(cfg:psvn_config_t; callback:svn_config_section_enumerator_t; baton:pointer):longint; cdecl; external libsvn_subr name 'svn_config_enumerate_sections';

type
  {* A callback function used in enumerating config sections.
   *
   * See svn_config_enumerate_sections2() for the details of this type.
   *
   * @since New in 1.3.
    }
  svn_config_section_enumerator2_t = function (name:pchar; baton:pointer; pool:apr_pool_p):svn_boolean_t;cdecl;
  
  {* Enumerate the sections, passing @a baton and the current section's name
   * to @a callback.  Continue the enumeration if @a callback returns @c TRUE.
   * Return the number of times @a callback was called. 
   *
   * ### See kff's comment to svn_config_enumerate2().  It applies to this
   * function, too. ###
   *
   * @a callback's @a name and @a name parameters are only valid for the
   * duration of the call.
   *
   * @since New in 1.3.
    }

  function svn_config_enumerate_sections2(cfg:psvn_config_t; callback:svn_config_section_enumerator2_t; baton:pointer; pool:apr_pool_p):longint; cdecl; external libsvn_subr name 'svn_config_enumerate_sections2';

type
  {* Similar to @c svn_config_enumerator2_t, but is not
   * provided with a memory pool argument.
   * See svn_config_enumerate() for the details of this type.
   *
   * @deprecated Provided for backwards compatibility with the 1.2 API. 
    }

  svn_config_enumerator_t = function (name:pchar; value:pchar; baton:pointer):svn_boolean_t;cdecl;
  {* Similar to svn_config_enumerate2(), but uses a memory pool of 
   * @a cfg instead of one that is explicitely provided.
   *
   * @deprecated Provided for backwards compatibility with the 1.2 API. 
    }

  function svn_config_enumerate(cfg:psvn_config_t; section:pchar; callback:svn_config_enumerator_t; baton:pointer):longint; cdecl; external libsvn_subr name 'svn_config_enumerate';

  type
  {* A callback function used in enumerating config options.
   *
   * See svn_config_enumerate2() for the details of this type.
   *
   * @since New in 1.3.
    }

  svn_config_enumerator2_t = function (name:pchar; value:pchar; baton:pointer; pool:apr_pool_p):svn_boolean_t;cdecl;

  {* Enumerate the options in @a section, passing @a baton and the current
   * option's name and value to @a callback.  Continue the enumeration if
   * @a callback returns @c TRUE.  Return the number of times @a callback 
   * was called.
   *
   * ### kff asks: A more usual interface is to continue enumerating
   *     while @a callback does not return error, and if @a callback does
   *     return error, to return the same error (or a wrapping of it)
   *     from svn_config_enumerate().  What's the use case for
   *     svn_config_enumerate()?  Is it more likely to need to break out
   *     of an enumeration early, with no error, than an invocation of
   *     @a callback is likely to need to return an error? ###
   *
   * @a callback's @a name and @a value parameters are only valid for the
   * duration of the call.
   *
   * @since New in 1.3.
    }

  function svn_config_enumerate2(cfg:psvn_config_t; section:pchar; callback:svn_config_enumerator2_t; baton:pointer; pool:apr_pool_p):longint; cdecl; external libsvn_subr name 'svn_config_enumerate2';

  {*
   * Return @c TRUE if @a section exists in @a config, @c FALSE otherwise.
   *
   * @since New in 1.4.
    }

  function svn_config_has_section(cfg:psvn_config_t; section:pchar):svn_boolean_t; cdecl; external libsvn_subr name 'svn_config_has_section';

  {* Enumerate the group @a master_section in @a cfg.  Each variable
   * value is interpreted as a list of glob patterns (separated by comma
   * and optional whitespace).  Return the name of the first variable
   * whose value matches @a key, or @c NULL if no variable matches.
    }

  function svn_config_find_group(cfg:psvn_config_t; key:pchar; master_section:pchar; pool:apr_pool_p):pchar; cdecl; external libsvn_subr name 'svn_config_find_group';

  {* Retrieve value corresponding to @a option_name for a given
   *  @a server_group in @a cfg , or return @a default_value if none is found.
   *
   *  The config will first be checked for a default, then will be checked for
   *  an override in a server group.
    }

  function svn_config_get_server_setting(cfg:psvn_config_t; server_group:pchar; option_name:pchar; default_value:pchar):pchar; cdecl; external libsvn_subr name 'svn_config_get_server_setting';

  {* Retrieve value into @a result_value corresponding to @a option_name for a
   *  given @a server_group in @a cfg, or return @a default_value if none is
   *  found.
   *
   *  The config will first be checked for a default, then will be checked for
   *  an override in a server group. If the value found is not a valid integer,
   *  a @c svn_error_t* will be returned.
    }

  function svn_config_get_server_setting_int(cfg:psvn_config_t; server_group:pchar; option_name:pchar; default_value:apr_int64_t; result_value:papr_int64_t; 
             pool:apr_pool_p):psvn_error_t; cdecl; external libsvn_subr name 'svn_config_get_server_setting_int';

  {* Try to ensure that the user's ~/.subversion/ area exists, and create
   * no-op template afiles for any absent config afiles.  Use @a pool for any
   * temporary allocation.  If @a config_dir is not @c NULL it specifies a
   * directory from which to read the config overriding all other sources.
   *
   * Don't error if something exists but is the wrong kind (for example,
   * ~/.subversion exists but is a afile, or ~/.subversion/servers exists
   * but is a directory).
   *
   * Also don't error if trying to create something and failing -- it's
   * okay for the config area or its contents not to be created.
   * However, if creating a config template afile succeeds, return an
   * error if unable to initialize its contents.
    }

  function svn_config_ensure(config_dir:pchar; pool:apr_pool_p):psvn_error_t; cdecl; external libsvn_subr name 'svn_config_ensure';

  {* Accessing cached authentication data in the user config area.
   *
   * @defgroup cached_authentication_data cached authentication data.
   * @
    }
  {* A hash-key pointing to a realmstring.  Every afile containing
   * authentication data should have this key.
    }
const
  SVN_CONFIG_REALMSTRING_KEY = 'svn:realmstring';     
  {* Use @a cred_kind and @a realmstring to locate a afile within the
   * ~/.subversion/auth/ area.  If the afile exists, initialize @a *hash
   * and load the afile contents into the hash, using @a pool.  If the
   * afile doesn't exist, set @a *hash to NULL.
   *
   * If @a config_dir is not NULL it specifies a directory from which to
   * read the config overriding all other sources.
   *
   * Besides containing the original credential fields, the hash will
   * also contain @c SVN_CONFIG_REALMSTRING_KEY.  The caller can examine
   * this value as a sanity-check that the correct afile was loaded.
   *
   * The hashtable will contain <tt>const char *</tt> keys and
   * <tt>svn_string_t *</tt> values.
    }

  function svn_config_read_auth_data(var hash: papr_hash_t; cred_kind:pchar; realmstring:pchar; config_dir:pchar; pool:apr_pool_p):psvn_error_t; cdecl; external libsvn_subr name 'svn_config_read_auth_data';

  {* Use @a cred_kind and @a realmstring to create or overwrite a afile
   * within the ~/.subversion/auth/ area.  Write the contents of @a hash into
   * the afile.  If @a config_dir is not NULL it specifies a directory to read
   * the config overriding all other sources.
   *
   * Also, add @a realmstring to the afile, with key @c
   * SVN_CONFIG_REALMSTRING_KEY.  This allows programs (or users) to
   * verify exactly which set credentials live within the afile.
   *
   * The hashtable must contain <tt>const char *</tt> keys and
   * <tt>svn_string_t *</tt> values.
    }

  function svn_config_write_auth_data(hash:papr_hash_t; cred_kind:pchar; realmstring:pchar; config_dir:pchar; pool:apr_pool_p):psvn_error_t; cdecl; external libsvn_subr name 'svn_config_write_auth_data';
