
(******************************************************************************
  svn_subr.inc

  Export of libsvn_subr

  Zolotov Alex, 2007
    mail: zolotov-alex@mail.ru
    url: http://zolotov.h14.ru

 *****************************************************************************)

  {* Put an English description of @a statcode into @a buf and return @a buf,
   * null-terminated. @a statcode is either an svn error or apr error.
    }
  function svn_strerror(statcode:apr_status_t; buf:Pchar; bufsize:apr_size_t):Pchar;cdecl;external libsvn_subr name 'svn_strerror';

  {* If @a err has a custom error message, return that, otherwise
   * store the generic error string associated with @a err->apr_err into
   * @a buf (terminating with null) and return @a buf.
   *
   * @since New in 1.4.
   *
   * @note @a buf and @a bufsize are provided in the interface so that
   * this function is thread-safe and yet does no allocation.
    }
  function svn_err_best_message(err:Psvn_error_t; buf:Pchar; bufsize:apr_size_t):Pchar;cdecl;external libsvn_subr name 'svn_err_best_message';

  {* Create a nested exception structure.
   *
   * Input:  an APR or SVN custom error code,
   *         a "child" error to wrap,
   *         a specific message
   *
   * Returns:  a new error structure (containing the old one).
   *
   * Notes: Errors are always allocated in a subpool of the global pool,
   *        since an error's lifetime is generally not related to the
   *        lifetime of any convenient pool.  Errors must be freed
   *        with svn_error_clear().  The specific message should be NULL
   *        if there is nothing to add to the general message associated
   *        with the error code.
   *
   *        If creating the "bottommost" error in a chain, pass @c NULL for
   *        the child argument.
    }
  function svn_error_create(apr_err:apr_status_t; child:Psvn_error_t; message:Pchar):Psvn_error_t;cdecl;external libsvn_subr name 'svn_error_create';

    {* A quick n' easy way to create a wrappered exception with your own
     * message, before throwing it up the stack.  (It uses all of the
     * child's fields.)
      }
  function svn_error_quick_wrap(child:Psvn_error_t; new_msg:Pchar):Psvn_error_t;cdecl;external libsvn_subr name 'svn_error_quick_wrap';

    {* Add @a new_err to the end of @a chain's chain of errors.  The @a new_err
     * chain will be copied into @a chain's pool and destroyed, so @a new_err
     * itself becomes invalid after this function.
      }
  procedure svn_error_compose(chain:Psvn_error_t; new_err:Psvn_error_t);cdecl;external libsvn_subr name 'svn_error_compose';

    {* Create a new error that is a deep copy of err and return it.
     *
     * @since New in 1.2.
      }
  function svn_error_dup(err:Psvn_error_t):Psvn_error_t;cdecl;external libsvn_subr name 'svn_error_dup';

    {* Free the memory used by @a error, as well as all ancestors and
     * descendants of @a error. 
     *
     * Unlike other Subversion objects, errors are managed explicitly; you 
     * MUST clear an error if you are ignoring it, or you are leaking memory. 
     * For convenience, @a error may be @c NULL, in which case this function does 
     * nothing; thus, svn_error_clear(svn_foo(...)) works as an idiom to 
     * ignore errors.
      }
  procedure svn_error_clear(error:Psvn_error_t);cdecl;external libsvn_subr name 'svn_error_clear';

    {*
     * Very basic default error handler: print out error stack @a error to the
     * stdio stream @a stream, with each error prefixed by @a prefix, and quit
     * iff the @a fatal flag is set.  Allocations are performed in the error's
     * pool.
     *
     * If you're not sure what prefix to pass, just pass "svn: ".  That's
     * what code that used to call svn_handle_error() and now calls
     * svn_handle_error2() does.
     *
     * @since New in 1.2.
      }
  procedure svn_handle_error2(error:Psvn_error_t; stream:PFILE; fatal:svn_boolean_t; prefix:Pchar);cdecl;external libsvn_subr name 'svn_handle_error2';

    {* Like svn_handle_error2() but with @c prefix set to "svn: "
     *
     * @deprecated Provided for backward compatibility with the 1.1 API.
      }
  procedure svn_handle_error(error:Psvn_error_t; stream:PFILE; fatal:svn_boolean_t);cdecl;external libsvn_subr name 'svn_handle_error';

    {*
     * Very basic default warning handler: print out the error @a error to the
     * stdio stream @a stream, prefixed by @a prefix.  Allocations are
     * performed in the error's pool.
     *
     * @since New in 1.2.
      }
  procedure svn_handle_warning2(stream:PFILE; error:Psvn_error_t; prefix:Pchar);cdecl;external libsvn_subr name 'svn_handle_warning2';

    {* Like svn_handle_warning2() but with @c prefix set to "svn: "
      }
  procedure svn_handle_warning(stream:PFILE; error:Psvn_error_t);cdecl;external libsvn_subr name 'svn_handle_warning';

    {* A statement macro for checking error values.
     *
     * Evaluate @a expr.  If it yields an error, return that error from the
     * current function.  Otherwise, continue.
     *
     * The <tt>do  ...  while (0)</tt> wrapper has no semantic effect,
     * but it makes this macro syntactically equivalent to the expression
     * statement it resembles.  Without it, statements like
     *
     * <tt>
     *   if (a)
     *     SVN_ERR (some operation);
     *   else
     *     foo;
     * </tt>
     *
     * would not mean what they appear to.
      }

  {* Wrapper around apr_pool_create_ex(), with a simpler interface.
   * The return pool will have an abort function set, which will call
   * abort() on OOM.
    }
  function svn_pool_create_ex(parent_pool:apr_pool_p; allocator:apr_allocator_p):apr_pool_p; cdecl; external libsvn_subr name 'svn_pool_create_ex';

  function svn_pool_create_ex_debug(parent_pool:apr_pool_p; allocator:apr_allocator_p; file_line:Pchar):apr_pool_p;cdecl;external libsvn_subr name 'svn_pool_create_ex_debug';
