program svnls;

{$mode objfpc}
{$longstrings on}
{$assertions on}

uses
  SysUtils, mxapr, mxsvn;

var
  allocator: apr_allocator_p;
  pool: apr_pool_p;
  err: svn_error_p;
  ctx: svn_client_ctx_p;
  rev: svn_opt_revision_t;
  dirents: apr_hash_p;
  hi: apr_hash_index_p;
  key, value: pointer;
  klen: apr_ssize_t;
  truepath: pchar;
  ab: psvn_auth_baton_t;
  cfg: svn_config_p;


procedure notify_func(baton: pointer;
       path: pchar; action: svn_wc_notify_action_t;
       kind: svn_node_kind_t; mime_type: pchar;
       content_state: svn_wc_notify_state_t;
       prop_state: svn_wc_notify_state_t;
       revision: svn_revnum_t); cdecl;
begin
  writeln('.');
end;

function cancel_func(cancel_baton: pointer): svn_error_p; cdecl;
begin
  writeln('.');
  Result := nil;
end;

function svn_ra_initialize(pool: apr_pool_p): svn_error_p; cdecl; external 'svn_ra-1' name 'svn_ra_initialize';
function svn_fs_initialize(pool: apr_pool_p): svn_error_p; cdecl; external 'svn_fs-1' name 'svn_fs_initialize';
function svn_opt_parse_path(rev: svn_opt_revision_p; truepath: ppchar; path: pchar; pool: apr_pool_p): svn_error_p; cdecl; external libsvn_subr name 'svn_opt_parse_path';

begin
  if ParamCount = 0 then
  begin
    writeln('svnls URL');
    Exit;
  end; // if
  
  svn_cmdline_init('svnls', nil);
  pool := svn_pool_create(nil);
  
  writeln(apr_allocator_create(allocator));
  assert(allocator <> nil);
  
  apr_allocator_max_free_set(allocator, SVN_ALLOCATOR_RECOMMENDED_MAX_FREE);
  
  pool := svn_pool_create_ex(nil, allocator);
  assert(pool <> nil);
  apr_allocator_owner_set(allocator, pool);
  
  err := svn_ra_initialize(pool);
  assert(err = nil);
  
  err := svn_config_ensure(nil, pool);
  assert(err = nil);
  
  err := svn_client_create_context(ctx, pool);
  assert(err = nil);
  
  err := svn_config_get_config(ctx^.config, nil, pool);
  assert(err = nil);
  
  cfg := apr_hash_get(ctx^.config, pchar(SVN_CONFIG_CATEGORY_CONFIG),
                     APR_HASH_KEY_STRING);

  
  ctx^.cancel_func := @cancel_func;
  
  err := svn_cmdline_setup_auth_baton(ab,
                                          true,
                                          nil,
                                          nil,
                                          nil,
                                          true,
                                          cfg,
                                          ctx^.cancel_func,
                                          ctx^.cancel_baton,
                                          pool);
  assert(err = nil);
    //svn_handle_error2(err, stderr, TRUE, "svn: ");

  ctx^.auth_baton := ab;
  //ctx^.notify_func := @notify_func;

  
  err := svn_opt_parse_path(@rev, @truepath, pchar(ParamStr(1)), pool);
  assert(err = nil);
  writeln('truepath: ', truepath);
  //writeln('rev: ', rev.value.number);
  
  writeln('do ls');
  rev.kind := svn_opt_revision_head;
  dirents := nil;
  err := svn_client_ls(dirents, truepath, @rev, false, ctx, pool);
  if ( err <> nil ) then
  begin
    writeln(err^.message);
  end; // if
  writeln('---------');
  assert(dirents <> nil);
  assert(err = nil);
  hi := apr_hash_first(pool, dirents);
  while hi <> nil do
  begin
    apr_hash_this(hi, key, klen, value);
    writeln(svn_dirent_p(value)^.last_author);
    writeln(copy(pchar(key), 1, klen));
    hi := apr_hash_next(hi);
  end; // while
  
  svn_pool_destroy(pool);
end.
