program svnlsauth;

{$mode objfpc}
{$longstrings on}
{$assertions on}

uses
  SysUtils, mxapr, mxsvn;

var
  allocator: apr_allocator_p;
  pool: apr_pool_p;
  err: svn_error_p;
  ctx: svn_client_ctx_p;
  rev: svn_opt_revision_t;
  dirents: apr_hash_p;
  hi: apr_hash_index_p;
  key, value: pointer;
  klen: apr_ssize_t;
  truepath: pchar;
  ab: psvn_auth_baton_t;
  cfg: svn_config_p;

function cancel_func(cancel_baton: pointer): svn_error_p; cdecl;
begin
  writeln('.');
  Result := nil;
end;

procedure push(providers: apr_array_header_p; provider: svn_auth_provider_object_p); inline;
begin
  ppointer(apr_array_push(providers))^ := provider;
end;

function simple_prompt(
    out cred: svn_auth_cred_simple_p;
    baton: pointer;
    realm: pchar;
    username: pchar;
    may_save: svn_boolean_t;
    pool: apr_pool_p
  ): svn_error_p; cdecl;
var s: string;
begin
  writeln('simple_prompt(', realm, ')');
  writeln('username: "', username, '"');
  cred := apr_palloc(pool, sizeof(cred^));
  FillChar(cred^, SizeOf(cred^), 0);
  
  if username <> nil 
    then cred^.username := apr_pstrdup(pool, username)
    else
    begin
      write('username: ');
      readln(s);
      cred^.username := apr_pstrdup(pool, pchar(s));
    end; // if
  
  write('password for ',cred^.username, ': ');
  readln(s);
  cred^.password := apr_pstrdup(pool, pchar(s));
  cred^.may_save := may_save;
  
  Result := nil;
end;

function username_prompt(
    out cred: svn_auth_cred_username_p;
    baton: pointer;
    realm: pchar;
    may_save: svn_boolean_t;
    pool: apr_pool_p
  ): svn_error_p; cdecl;
var s: string;
begin
  writeln('username_prompt(', realm, ')');
  cred := apr_palloc(pool, sizeof(cred^));
  FillChar(cred^, SizeOf(cred^), 0);
  write('username: ');
  readln(s);
  cred^.username := apr_pstrdup(pool, pchar(s));
  cred^.may_save := may_save;
  Result := nil;
end;

procedure setup_auth;
var
  provider: svn_auth_provider_object_p;
  providers: apr_array_header_p;
  store_password_val: svn_boolean_t;
begin
  providers := apr_array_make(pool, 12, sizeof(svn_auth_provider_object_p));
  
  svn_auth_get_simple_provider(provider, pool);
  push(providers, provider);
  
  svn_auth_get_username_provider(provider, pool);
  push(providers, provider);
  
  (* The server-cert, client-cert, and client-cert-password providers. *)
  svn_auth_get_ssl_server_trust_file_provider(provider, pool);
  push(providers, provider);
  
  svn_auth_get_ssl_client_cert_file_provider(provider, pool);
  push(providers, provider);
  
  svn_auth_get_ssl_client_cert_pw_file_provider(provider, pool);
  push(providers, provider);
  
  (* Two basic prompt providers: username/password, and just username. *)
  svn_auth_get_simple_prompt_provider(provider, @simple_prompt, nil, 2, pool);
  push(providers, provider);
  
  svn_auth_get_username_prompt_provider(provider, @username_prompt, nil, 2, pool);
  push(providers, provider);

  svn_auth_open(ab, providers, pool);
  ctx^.auth_baton := ab;
  
  assert(svn_config_get_bool(cfg, store_password_val, SVN_CONFIG_SECTION_AUTH, SVN_CONFIG_OPTION_STORE_PASSWORDS, true) = nil);
  writeln('store_password_val: ', store_password_val);
  
  if not store_password_val then
    svn_auth_set_parameter(ab, SVN_AUTH_PARAM_DONT_STORE_PASSWORDS, pchar(''));
  
  (* There are two different ways the user can disable disk caching
     of credentials:  either via --no-auth-cache, or in the config
     file ('store-auth-creds = no'). *)
  assert(svn_config_get_bool(cfg, store_password_val,
                              SVN_CONFIG_SECTION_AUTH,
                              SVN_CONFIG_OPTION_STORE_AUTH_CREDS,
                              true) = nil);

  if not store_password_val then
    svn_auth_set_parameter(ab, SVN_AUTH_PARAM_NO_AUTH_CACHE, pchar(''));
end;

function svn_ra_initialize(pool: apr_pool_p): svn_error_p; cdecl; external 'svn_ra-1' name 'svn_ra_initialize';
function svn_fs_initialize(pool: apr_pool_p): svn_error_p; cdecl; external 'svn_fs-1' name 'svn_fs_initialize';
function svn_opt_parse_path(rev: svn_opt_revision_p; truepath: ppchar; path: pchar; pool: apr_pool_p): svn_error_p; cdecl; external libsvn_subr name 'svn_opt_parse_path';

begin
  if ParamCount = 0 then
  begin
    writeln('svnls URL');
    Exit;
  end; // if
  
  svn_cmdline_init('svnls', nil);
  pool := svn_pool_create(nil);
  
  assert(apr_allocator_create(allocator) = 0);
  assert(allocator <> nil);
  
  apr_allocator_max_free_set(allocator, SVN_ALLOCATOR_RECOMMENDED_MAX_FREE);
  
  pool := svn_pool_create_ex(nil, allocator);
  assert(pool <> nil);
  apr_allocator_owner_set(allocator, pool);
  
  err := svn_ra_initialize(pool);
  assert(err = nil);
  
  err := svn_config_ensure(nil, pool);
  assert(err = nil);
  
  err := svn_client_create_context(ctx, pool);
  assert(err = nil);
  
  err := svn_config_get_config(ctx^.config, nil, pool);
  assert(err = nil);
  
  cfg := apr_hash_get(ctx^.config, pchar(SVN_CONFIG_CATEGORY_CONFIG),
                     APR_HASH_KEY_STRING);
  
  ctx^.cancel_func := @cancel_func;
  
  setup_auth;
  
  err := svn_opt_parse_path(@rev, @truepath, pchar(ParamStr(1)), pool);
  assert(err = nil);
  writeln('truepath: ', truepath);
  //writeln('rev: ', rev.value.number);
  
  writeln('do ls');
  rev.kind := svn_opt_revision_head;
  dirents := nil;
  err := svn_client_ls(dirents, truepath, @rev, false, ctx, pool);
  if ( err <> nil ) then
  begin
    writeln(err^.message);
  end; // if
  writeln('---------');
  assert(dirents <> nil);
  assert(err = nil);
  hi := apr_hash_first(pool, dirents);
  while hi <> nil do
  begin
    apr_hash_this(hi, key, klen, value);
    writeln(copy(pchar(key), 1, klen));
    hi := apr_hash_next(hi);
  end; // while
  
  svn_pool_destroy(pool);
end.
