unit pmsclient;

{$IFDEF FPC}
 {$MODE DELPHI}
{$ENDIF}
{$ASSERTIONS ON}

interface
uses SysUtils, Classes, mxvalues, mxrpc;

type
  TPMSClient = class (TMXRPC)
    public
      (**
       * @rpc-param $limit максимальное число проектов
       *  если 0 или меньше нуля, то выбарать столько, сколько может выдать сервер
       * @rpc-param $offset число проектов которые нужно пропустить
       * @return true, если операция успешна, результат в Response
       * @note проекты сортируются по дате регистрации
       * @note система ограничивает максимальное число выдаваемых проектов,
       *   поэтому в результате может быть меньше проектов, чем запрошено
       *)
      function select_projects(limit: Integer; offset: Integer = 0): Boolean;
      
      function get_project_by_id(ID: Integer): Boolean;
      function get_project_by_name(const Name: UTF8String): Boolean;
  end;

implementation

function TPMSClient.select_projects(limit: Integer;  offset: Integer): Boolean;
begin
  Request.Clear;
  Request.AddInteger(limit);
  Request.AddInteger(offset);
  Result := Invoke('pms.select_projects') and not IsFault;
end;

function TPMSClient.get_project_by_id(ID: Integer): Boolean;
begin
  Request.Clear;
  Request.AddInteger(ID);
  Result := Invoke('pms.get_project_info') and not IsFault;
end;

function TPMSClient.get_project_by_name(const Name: UTF8String): Boolean;
begin
  Request.Clear;
  Request.AddString(Name);
  Result := Invoke('pms.get_project_info') and not IsFault;
end;


end.
