
{$IFDEF FPC}
 {$MODE DELPHI}
{$ENDIF}
{$ASSERTIONS ON}

uses SysUtils, Classes, mxvalues, pmsclient, DateUtils, IniFiles;

(** тест метода select_projects *)
procedure test_select_projects(pms: TPMSClient);
const
  // ограничение на число проектов выбираемых за раз
  // если 0 или меньше нуля, то выбарать столько, сколько
  // может выдать сервер
  limit = 0; 
var
  List: TMXArrayValue; // список полученых проектов
  total: Integer; // общее число зарегистрированных проектов
  offset: Integer; // число проектов которые мы уже обработали
  
  procedure list_projects(List: TMXArrayValue);
  var i, count: Integer;
  begin
    count := List.Count;
    for i := 0 to count-1 do
    begin
      writeln('------------------------');
      with List.AsRecord[i] do
      begin
        writeln('ID:       ', AsInteger['id']);
        writeln('Name:     ', AsString['name']);
        writeln('Title:    ', AsString['title']);
        writeln('RegTime:  ', DateTimeToStr(UnixToDateTime(AsInteger['regtimestamp'])));
        writeln;
      end; // with
    end; // for i
  end;
begin
  if not pms.select_projects(limit, 0) then
  begin
    writeln('fault ', pms.FaultCode, ': ', pms.FaultString);
    exit;
  end; // if
  List := pms.Response.AsRecord.AsArray['projects'];
  total := pms.Response.AsRecord.AsInteger['total'];
  offset := List.Count;
  
  writeln('total registered projects: ', total);
  writeln('selected projects:');
  list_projects(List);
  
  while offset < total do
  begin
    if not pms.select_projects(limit, offset) then
    begin
      writeln('fault ', pms.FaultCode, ': ', pms.FaultString);
      exit;
    end; // if
    // обновляем total: мало ли, может число проектов стало меньше,
    // тогда мы попадем в бесконечный цикл
    total := pms.Response.AsRecord.AsInteger['total'];
    List := pms.Response.AsRecord.AsArray['projects'];
    list_projects(List);
    offset := offset + List.Count;
  end; // while
end;

(** отобразить описание проекта *)
procedure ShowProjectInfo(Prj: TMXRecordValue);
begin
  writeln('ID:       ', prj.AsInteger['id']);
  writeln('Name:     ', prj.AsString['name']);
  writeln('Title:    ', prj.AsString['title']);
  writeln('RegTime:  ', DateTimeToStr(UnixToDateTime(prj.AsInteger['regtimestamp'])));
  writeln('Description:');
  writeln('--------- START OF DESCRIPTION --------');
  writeln(prj.AsString['description']);
  writeln('--------- END OF DESCRIPTION ----------');
end;

(** тест метода get_project_by_id *)
procedure test_project_by_id(pms: TPMSClient; ID: Integer);
begin
  if not pms.get_project_by_id(ID) then
  begin
    writeln('fault ', pms.FaultCode, ': ', pms.FaultString);
    exit;
  end; // if
  
  // отобразить описание проекта
  ShowProjectInfo(pms.Response.AsRecord);
end;

(** тест метода get_project_by_name *)
procedure test_project_by_name(pms: TPMSClient; const Name: UTF8String);
begin
  if not pms.get_project_by_name(Name) then
  begin
    writeln('fault ', pms.FaultCode, ': ', pms.FaultString);
    exit;
  end; // if
  
  // отобразить описание проекта
  ShowProjectInfo(pms.Response.AsRecord);
end;

procedure test(pms: TPMSClient);
var cmd: string;
begin
  if ParamCount > 0 then
  begin
    cmd := ParamStr(1);
    if cmd = 'list' then
    begin
      test_select_projects(pms);
      Exit;
    end
    else if cmd = 'project_by_id' then
    begin
      test_project_by_id(pms, StrToInt(ParamStr(2)));
      Exit;
    end
    else if cmd = 'project_by_name' then
    begin
      test_project_by_name(pms, ParamStr(2));
      Exit;
    end;
  end;
  writeln('test for pms client');
  writeln('commands:');
  writeln('  testpms list - list projects');
  writeln('  testpms project_by_id <ID> - show project info by ID');
  writeln('  testpms project_by_name <Name> - show project info by Name');
end;

var
  ini: TIniFile;
  pms: TPMSClient;
begin
  ini := TIniFile.Create('pmsclient.ini');
  pms := TPMSClient.Create(ini.ReadString('pmsclient', 'gate', 'http://localhost/pms/trunc/rpc/pms.php'));
  ini.Free;
  try
    test(pms);
    writeln;
    writeln('RPC query count: ', pms.QueryCount);
  except
    on E: Exception do
    begin
      writeln(stderr);
      writeln(stderr, E.ClassName, ': ', E.Message);
    end;
  end;
  pms.Free;
end.
